import { ArcGisMapServerImageryProvider } from 'cesium';
import { BingMapsImageryProvider } from 'cesium';
import { BingMapsStyle } from 'cesium';
import * as Cesium from 'cesium';
import { Ellipsoid } from 'cesium';
import { ImageryLayer } from 'cesium';
import { ImageryProvider } from 'cesium';
import { OpenStreetMapImageryProvider } from 'cesium';
import { Rectangle } from 'cesium';
import { SingleTileImageryProvider } from 'cesium';
import { TileDiscardPolicy } from 'cesium';
import { TilingScheme } from 'cesium';

/**
 * ### 基础类
 */
declare class BaseClass {
    /**
     * 当前类的构造参数
     */
    options?: any;
    constructor(options?: any);
    destroy(): void;
    /**
     * ### 绑定指定类型事件监听器
     * ---
     * @param type 事件类型
     * @param callback 回调函数
     * @param [context] 侦听器上下文 this 的指向
     * @param [once] 是否为单次监听
     * @returns 当前实例, 可用于链式调用
     */
    on(): BaseClass;
    /**
     * ### 解除绑定指定类型事件监听器
     * ---
     * @param type 事件类型
     * @param [callback] 回调函数
     * @param [context] 侦听器上下文 this 的指向
     * @returns 当前实例, 可用于链式调用
     */
    off(): BaseClass;
}

/**
 * ### 地图基础类
 * ---
 * @param id - 地图div容器的id 或 已构造好的Viewer对象
 * @param [options = {}] - 地图基础类的构造参数
 */
export declare class BaseMap extends BaseClass {
    /**
     * 地图容器id
     */
    id: string;
    private imageryProviderViewModels;
    /**
     * ### 地图基础类的构造参数
     */
    readonly options: MapType.Options;
    /**
     * ### Cesium原生 Viewer 对象
     * ---
     * @link [Cesium.Viewer](https://cesium.com/learn/cesiumjs/ref-doc/Viewer.html)
     */
    viewer?: Cesium.Viewer;
    constructor(id: string, options?: MapType.Options);
    private init;
    private createViewer;
    private createImageryProviderViewModels;
    private createBaseLayer;
    /**
     * 获取当前选择的图层
     */
    private getSelectedImageryProviderViewModel;
    private initLayer;
    private setToken;
    destroy(): void;
}

export declare const layer: {
    /**
     * 基础图层
     */
    BaseLayer: {
        new (options: LayerType.Layer): {
            id: string;
            name: string;
            show: boolean;
            type: LayerType.LayerEnum;
            _layer: ImageryLayer | undefined;
            _map: BaseMap | undefined;
            layer: ImageryLayer | undefined;
            addToMap(map: BaseMap): any;
            options?: any;
            destroy(): void;
            on(): BaseClass;
            off(): BaseClass;
        };
    };
    /**
     * 单张图片图层
     */
    ImageLayer: {
        new (options: LayerType.ImageLayer & LayerType.ImageLayerOptions): {
            id: string;
            name: string;
            show: boolean;
            type: LayerType.LayerEnum;
            _layer: ImageryLayer | undefined;
            _map: BaseMap | undefined;
            layer: ImageryLayer | undefined;
            addToMap(map: BaseMap): any;
            options?: any;
            destroy(): void;
            on(): BaseClass;
            off(): BaseClass;
        };
    };
    /**
     * 必应图层
     */
    BingLayer: {
        new (options: LayerType.BingLayer & LayerType.ImageLayerOptions): {
            id: string;
            name: string;
            show: boolean;
            type: LayerType.LayerEnum;
            _layer: ImageryLayer | undefined;
            _map: BaseMap | undefined;
            layer: ImageryLayer | undefined;
            addToMap(map: BaseMap): any;
            options?: any;
            destroy(): void;
            on(): BaseClass;
            off(): BaseClass;
        };
    };
    /**
     * ArcGis图层
     */
    ArcGisLayer: {
        new (options: LayerType.ArcGISLayer & LayerType.ImageLayerOptions): {
            id: string;
            name: string;
            show: boolean;
            type: LayerType.LayerEnum;
            _layer: ImageryLayer | undefined;
            _map: BaseMap | undefined;
            layer: ImageryLayer | undefined;
            addToMap(map: BaseMap): any;
            options?: any;
            destroy(): void;
            on(): BaseClass;
            off(): BaseClass;
        };
    };
    /**
     * OSM图层
     */
    OSMLayer: {
        new (options: LayerType.OSMLayer & LayerType.ImageLayerOptions): {
            id: string;
            name: string;
            show: boolean;
            type: LayerType.LayerEnum;
            _layer: ImageryLayer | undefined;
            _map: BaseMap | undefined;
            layer: ImageryLayer | undefined;
            addToMap(map: BaseMap): any;
            options?: any;
            destroy(): void;
            on(): BaseClass;
            off(): BaseClass;
        };
    };
    /**
     * 图层工具类
     */
    LayerUtil: {
        new (): {};
        createSingleTileImageryProvider(options: LayerType.ImageLayer): SingleTileImageryProvider;
        createBingMapsImageryProvider(options?: LayerType.BingLayer): Promise<BingMapsImageryProvider>;
        crerateArcGisMapServerImageryProvider(options: LayerType.ArcGISLayer): Promise<ArcGisMapServerImageryProvider>;
        createOsmImageryProvider(options: LayerType.OSMLayer): OpenStreetMapImageryProvider;
        createImageryLayerProvider(options: LayerType.AllLayer): Promise<ImageryProvider>;
        createBaseLayer(options: LayerType.AllLayer): {
            id: string;
            name: string;
            show: boolean;
            type: LayerType.LayerEnum;
            _layer: ImageryLayer | undefined;
            _map: BaseMap | undefined;
            layer: ImageryLayer | undefined;
            addToMap(map: BaseMap): any;
            options?: any;
            destroy(): void;
            on(): BaseClass;
            off(): BaseClass;
        } | {
            id: string;
            name: string;
            show: boolean;
            type: LayerType.LayerEnum;
            _layer: ImageryLayer | undefined;
            _map: BaseMap | undefined;
            layer: ImageryLayer | undefined;
            addToMap(map: BaseMap): any;
            options?: any;
            destroy(): void;
            on(): BaseClass;
            off(): BaseClass;
        } | {
            id: string;
            name: string;
            show: boolean;
            type: LayerType.LayerEnum;
            _layer: ImageryLayer | undefined;
            _map: BaseMap | undefined;
            layer: ImageryLayer | undefined;
            addToMap(map: BaseMap): any;
            options?: any;
            destroy(): void;
            on(): BaseClass;
            off(): BaseClass;
        } | {
            id: string;
            name: string;
            show: boolean;
            type: LayerType.LayerEnum;
            _layer: ImageryLayer | undefined;
            _map: BaseMap | undefined;
            layer: ImageryLayer | undefined;
            addToMap(map: BaseMap): any;
            options?: any;
            destroy(): void;
            on(): BaseClass;
            off(): BaseClass;
        };
        _createImageLayerOptions(options: LayerType.AllLayer & LayerType.ImageLayerOptions): {
            show: boolean;
            rectangle: Rectangle;
            type: LayerType.LayerEnum.Image;
            url: string;
            tileWidth?: number;
            tileHeight?: number;
            credit?: string;
            id?: string;
            name?: string;
            alpha?: number;
            nightAlpha?: number;
            dayAlpha?: number;
            brightness?: number;
            contrast?: number;
            hue?: number;
            saturation?: number;
            gamma?: number;
        } | {
            show: boolean;
            rectangle: Rectangle;
            type: LayerType.LayerEnum.Bing;
            key?: string;
            mapStyle?: BingMapsStyle;
            culture?: string;
            ellipsoid?: Ellipsoid;
            tileDiscardPolicy?: TileDiscardPolicy;
            id?: string;
            name?: string;
            alpha?: number;
            nightAlpha?: number;
            dayAlpha?: number;
            brightness?: number;
            contrast?: number;
            hue?: number;
            saturation?: number;
            gamma?: number;
        } | {
            show: boolean;
            rectangle: Rectangle;
            type: LayerType.LayerEnum.ArcGIS;
            tileDiscardPolicy?: TileDiscardPolicy;
            usePreCachedTilesIfAvailable?: boolean;
            layers?: string;
            enablePickFeatures?: boolean;
            tilingScheme?: TilingScheme;
            ellipsoid?: Ellipsoid;
            credit?: string;
            tileWidth?: number;
            tileHeight?: number;
            maximumLevel?: number;
            minimumLevel?: number;
            token?: string;
            id?: string;
            name?: string;
            alpha?: number;
            nightAlpha?: number;
            dayAlpha?: number;
            brightness?: number;
            contrast?: number;
            hue?: number;
            saturation?: number;
            gamma?: number;
        } | {
            show: boolean;
            rectangle: Rectangle;
            type: LayerType.LayerEnum.OSM;
            url?: string;
            fileExtension?: string;
            retinaTiles?: boolean;
            minimumLevel?: number;
            maximumLevel?: number;
            ellipsoid?: Ellipsoid;
            credit?: string;
            id?: string;
            name?: string;
            alpha?: number;
            nightAlpha?: number;
            dayAlpha?: number;
            brightness?: number;
            contrast?: number;
            hue?: number;
            saturation?: number;
            gamma?: number;
        };
    };
    /**
     * 地形工具类
     */
    TerrainUtil: {
        new (): {};
        createTerrainProvider(): void;
    };
};

export declare namespace LayerType {
    export type AllLayer = (ImageLayer | BingLayer | ArcGISLayer | OSMLayer) & ImageLayerOptions;
    /**
     * 图片覆盖的矩形范围；默认为全球范围
     * @param xmin 最小经度值, -180 至 180
     * @param xmax 最大经度值, -180 至 180
     * @param ymin 最小纬度值, -90 至 90
     * @param ymax 最大纬度值, -90 至 90
     * @default Cesium.Rectangle.MAX_VALUE
     * @description Cesium.Rectangle(west, south, east, north)
     * @link https://cesium.com/learn/cesiumjs/ref-doc/Rectangle.html?classFilter=Rectangle
     */
    export type Rectangle = [xmin: number, xmax: number, ymin: number, ymax: number];
    /**
     * 地图图层枚举
     */
    export enum LayerEnum {
        /**
         * 单张图片
         */
        Image = "image",
        /**
         * 微软Bing地图
         */
        Bing = "bing",
        /**
         * ArcGIS地图
         */
        ArcGIS = "arcgis",
        /**
         * OpenStreetMap地图
         */
        OSM = "osm"
    }
    /**
     * 基础图层配置
     */
    export type BaseMap = {
        /**
         * 图层名称
         */
        name?: string;
        /**
         * 图层鼠标悬浮提示
         */
        tooltip?: string;
        /**
         * 图层图标,用于图层列表显示
         */
        iconUrl?: string;
        /**
         * 是否选中为默认地图
         */
        selected?: boolean;
        /**
         * 图层创建函数
         */
        [property: string]: any;
    };
    export type ImageLayerOptions = {
        rectangle?: Rectangle;
        /**
         * 图片透明度 0-1
         * @default 1.0
         */
        alpha?: number;
        /**
         * 图层在地球夜间一侧的透明度 0-1
         * @default 1.0
         */
        nightAlpha?: number;
        /**
         * 图层在地球白天一侧的透明度 0-1
         * @default 1.0
         */
        dayAlpha?: number;
        /**
         * 图层亮度
         * @default 1.0
         */
        brightness?: number;
        /**
         * 图层对比度
         * @default 1.0
         */
        contrast?: number;
        /**
         * 图层色调
         * @default 0.0
         */
        hue?: number;
        /**
         * 图层饱和度
         * @default 1.0
         */
        saturation?: number;
        /**
         * 图层伽马校正
         * @default 1.0
         */
        gamma?: number;
        /**
         * 图层是否显示
         * @default true 默认显示
         */
        show?: boolean;
    };
    /**
     * 地图图层基础配置
     */
    export type Layer = {
        /**
         * 图层id标识
         */
        id?: string;
        /**
         * 图层名称
         */
        name?: string;
        /**
         * 图层类型
         */
        readonly type: LayerEnum;
        /**
         * 图层是否显示
         * @default true 默认显示
         */
        show?: boolean;
        /**
         * 覆盖的矩形范围；默认为全球范围
         */
        rectangle?: Rectangle;
    };
    /**
     * 单张图片图层配置
     */
    export type ImageLayer = {
        /**
         * 单张图片图层类型
         */
        type: LayerEnum.Image;
        /**
         * 图片地址
         */
        url: string;
        /**
         * 图片宽度, 以像素为单位
         */
        tileWidth?: number;
        /**
         * 图片高度, 以像素为单位
         */
        tileHeight?: number;
        /**
         * 版权信息
         */
        credit?: string;
    } & Layer;
    /**
     * Bing地图配置
     * @link https://www.bingmapsportal.com/
     */
    export type BingLayer = {
        /**
         * 必应地图图层
         */
        readonly type: LayerEnum.Bing;
        /**
         * 必应地图秘钥
         */
        key?: string;
        /**
         * 必应地图风格
         * @default Cesium.BingMapsStyle.AERIAL
         * @link https://cesium.com/learn/cesiumjs/ref-doc/global.html#BingMapsStyle
         */
        mapStyle?: Cesium.BingMapsStyle;
        /**
         * 请求必应地图图像时要使用的区域性
         * @link http://msdn.microsoft.com/en-us/library/hh441729.aspx
         */
        culture?: string;
        /**
         * 椭球体，默认为WGS84
         */
        ellipsoid?: Cesium.Ellipsoid;
        /**
         * 确定图块是否无效并应丢弃的策略
         * @description 默认情况下，将使用`DiscardEmptyTileImagePolicy` ，并期望 Bing Maps 服务器将针对缺失的图块发送零长度响应。为确保不丢弃任何图块，请NeverTileDiscardPolicy为此参数构造并传递一个。
         */
        tileDiscardPolicy?: Cesium.TileDiscardPolicy;
    } & Layer;
    /**
     * ArcGIS图层配置
     * @link https://developers.arcgis.com/
     */
    export type ArcGISLayer = {
        /**
         * ArcGIS图层类型
         */
        readonly type: LayerEnum.ArcGIS;
        /**
         * 确定图块是否无效并应丢弃的策略
         */
        tileDiscardPolicy?: Cesium.TileDiscardPolicy;
        /**
         * 是否使用预缓存的瓦片
         * @default true
         */
        usePreCachedTilesIfAvailable?: boolean;
        /**
         * 要显示的图层的逗号分隔列表, 默认为所有图层
         */
        layers?: string;
        enablePickFeatures?: boolean;
        /**
         * 覆盖的矩形范围；默认为全球范围
         */
        rectangle?: Rectangle;
        /**
         * 将世界划分为图块的图块方案
         */
        tilingScheme?: Cesium.TilingScheme;
        ellipsoid?: Cesium.Ellipsoid;
        /**
         * 版权信息
         */
        credit?: string;
        /**
         * 图片宽度, 以像素为单位
         */
        tileWidth?: number;
        /**
         * 图片高度, 以像素为单位
         */
        tileHeight?: number;
        /**
         * 请求的最大图块级别
         */
        maximumLevel?: number;
        /**
         * 请求的最小图块级别
         */
        minimumLevel?: number;
        /**
         * ArcGIS服务token
         */
        token?: string;
    } & Layer;
    /**
     * OSM地图配置
     * @link https://wiki.openstreetmap.org/
     */
    export type OSMLayer = {
        /**
         * OSM地图图层类型
         */
        readonly type: LayerEnum.OSM;
        /**
         * osm地图服务地址
         * @default https://tile.openstreetmap.org
         */
        url?: string;
        /**
         * 图片扩展名
         * @default png
         */
        fileExtension?: string;
        /**
         * 是否支持视网膜分辨率
         * @default false
         * @description 如果为真，则请求以 2 倍分辨率显示视网膜屏幕的图块
         */
        retinaTiles?: boolean;
        /**
         * 图像提供商支持的最低细节级别
         */
        minimumLevel?: number;
        /**
         * 图像提供商支持的最大细节级别
         */
        maximumLevel?: number;
        /**
         * 椭球体，默认为WGS84
         */
        ellipsoid?: Cesium.Ellipsoid;
        /**
         * 版权信息
         */
        credit?: string;
    } & Layer;
}

export declare namespace MapType {
    /**
     * 地图初始化配置
     */
    export type Options = {
        /**
         * 地图控件配置
         */
        control?: Control;
        /**
         * 场景配置
         */
        scene?: Scene;
        /**
         * 可叠加图层配置
         */
        baseMaps?: LayerType.BaseMap[];
        /**
         * 可叠加图层配置
         */
        layers?: LayerType.AllLayer[];
        /**
         * 地球的最底层影像层。如果设置为false，则不会添加任何影像提供商。此值仅在`baseLayerPicker`设置为 `false` 时有效
         */
        baseLayer?: false | LayerType.AllLayer;
        /**
         * 各服务商token配置
         */
        token?: Token;
    };
    /**
     * 地图控件配置
     */
    export interface Control {
        /**
         * 是否创建动画部件
         * @default false
         */
        animation?: boolean;
        /**
         * 是否显示底图切换按钮
         * @default false
         */
        baseLayerPicker?: boolean;
        /**
         * 是否显示全屏按钮
         * @default false
         */
        fullscreenButton?: boolean;
        /**
         * 是否显示VR按钮
         * @default false
         */
        vrButton?: boolean;
        /**
         * 是否显示地名查找组件
         * @default false
         */
        geocoder?: boolean;
        /**
         * 是否显示视角复位按钮
         * @default false
         */
        homeButton?: boolean;
        /**
         * 是否显示点击要素之后显示的信息面板
         * @default false
         */
        infoBox?: boolean;
        /**
         * 是否显示二维、三维、2.5D视图切换按钮
         * @default false
         */
        sceneModePicker?: boolean;
        /**
         * 是否显示选择模型时的绿色框
         * @default false
         */
        selectionIndicator?: boolean;
        /**
         * 是否创建下侧时间线控件面板
         * @default false
         */
        timeline?: boolean;
        /**
         * 是否显示帮助按钮
         * @default false
         */
        navigationHelpButton?: boolean;
        /**
         * 在用户明确单击按钮之前是否自动显示帮助按钮
         * @default false
         */
        navigationInstructionsInitiallyVisible?: boolean;
        /**
         * 时钟视图模型
         */
        clockViewModel?: Cesium.ClockViewModel;
    }
    /**
     * 地图场景配置
     */
    export interface Scene {
        /**
         * 几何实例是否将仅以3D渲染，以节省GPU内存
         * @default false
         */
        scene3DOnly?: boolean;
        /**
         * 是否开启时钟动画
         * @default true
         */
        shouldAnimate?: boolean;
    }
    /**
     * 地图token配置
     */
    export type Token = {
        /**
         * Cesium ion地图token
         */
        ion?: string;
        /**
         * 腾讯地图token
         */
        tencent?: string;
        /**
         * 高德地图token
         */
        amap?: string;
        /**
         * 天地图token
         */
        tianditu?: string;
        /**
         * 百度地图token
         */
        baidu?: string;
        /**
         * 微软Bing地图
         */
        bing?: string;
        /**
         * 微软Bing地图
         */
        arcgis?: string;
    };
}

export declare const Tools: {
    uuid: () => string;
};

export { }
